/*
 *    driver_FDC.h - generic FDC driver for WD279x (Z80)
 *
 *    Copyright (c) 2012 by Michal Hucik <http://www.ordoz.com>
 *
 *    Thies program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    driver_FDC is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with driver_FDC; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


#ifndef _driver_FDC
#define _driver_FDC


#define FDC_DRIVE0              0x00
#define FDC_DRIVE1              0x01
#define FDC_DRIVE2              0x02
#define FDC_DRIVE3              0x03


typedef unsigned char FDC_DRIVE;


/* definice parametru media - vastavuje se v FDC_DRIVE */
#define FDC_SECTOR_SIZE_128	0x00
#define FDC_SECTOR_SIZE_256	0x01
#define FDC_SECTOR_SIZE_512	0x02
#define FDC_SECTOR_SIZE_1024	0x03
#define FDC_MEDIUM_SIDES1	( 0 << 2 )
#define FDC_MEDIUM_SIDES2	( 1 << 2 )
#define FDC_MEDIUM_NOT_CPL	( 0 << 3 )
#define FDC_MEDIUM_CPL		( 1 << 3 )
#define FDC_MEDIUM_DD		( 0 << 4 )
#define FDC_MEDIUM_HD		( 1 << 4 )	/* jen u HD ready radice */


/* verze driveru/radice */

typedef struct {
    unsigned int major;
    unsigned int minor;
    char *description;
} FDC_VERSION;



/*
 * Konfigurace timingu
 *
*/

typedef struct {
    unsigned int driveOnTime;   /* Drive start time in ms */
    unsigned char trackTime;    /* Track setting time in ms */
    unsigned char stepTime;     /* Single step time in ms */
    unsigned char motorPredef;	/* init value for _FDC_Motor */
} FDC_TIMERS;
            
            


/* Nenulova hodnota znaci bezici motor.
 *
 * System si bude v interruptu decrementovat hodnotu v _FDC_Motor.
 * Jakmile ji vynuluje, tak zavola FDC_serviceMotorOff
 *
*/
extern unsigned char _FDC_Motor;



/* sluzby driveru */

extern FDC_VERSION* FDC_serviceVersion ( void );
extern FDC_TIMERS* FDC_serviceTimers ( void );
extern FDC_DRIVE* FDC_serviceSeldsk ( unsigned char drive_id );

/*
 * Nastaveni stopy.
 *
 * Vstupem je logicke cislo stopy.
 * 
 * log.trk 0 = track: 0, side: 0
 * log.trk 1 = track: 0, side: 1
 * log.trk 2 = track: 1, side: 0
 * ...
 * ...
*/
extern void FDC_serviceSettrk ( unsigned char track );

/*
 * Nastaveni fyzickeho sektoru.
*/
extern void FDC_serviceSetsec ( unsigned char sector );

/*
 * Adresa na kterou se ukladaji ctena data, 
 * nebo ze ktere se ctou zapisovana data.
*/
extern void FDC_serviceSetdma ( void* dma );

/*
 * Nasilne vypnuti motoru.
*/
extern void FDC_serviceMotorOff ( void  );

/*
 * Pecteni nastaveneho sektoru
 *
 * Vraci:
 *		 0: OK
 *		-1: ERROR
*/
extern char FDC_serviceRead ( void );

/*
 * Zapis nastaveneho sektoru
 *
 * Vraci:
 *		 0: OK
 *		-1: ERROR
 *		-2: WRITE PROTECTED
*/
extern char FDC_serviceWrite ( void );



/* sluzby, ktere nam poskytuje system */
extern void (*_FDC_serviceDI) ( void );
extern void (*_FDC_serviceEI) ( void );

#endif
