/*
 *    main.c - generic FDC driver for WD279x (Z80)
 *
 *    Copyright (c) 2012 by Michal Hucik <http://www.ordoz.com>
 *
 *    Thies program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    driver_FDC is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with driver_FDC; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <stdio.h>

#include "video.h"
#include "monitor.h"

#include "test_FDC.h"
#include "driver_FDC.h"


//#define USE_WRITE_TEST


__sfr __at 		0xce 	rDMD;

// porty k mapovani standardni pameti
__sfr __at 		0xe0 	rMAPE0;
__sfr __at 		0xe1 	rMAPE1;
__sfr __at 		0xe2 	rMAPE2;
__sfr __at 		0xe3 	rMAPE3;
__sfr __at 		0xe4 	rMAPE4;
__sfr __at 		0xe5 	rMAPE5;
__sfr __at 		0xe6 	rMAPE6;



void prg_error ( unsigned char err_code ) {
    xprintf ( " ERR = %d", err_code );
    while ( 1 ) {
        rBCOLOR = err_code;
        rBCOLOR = 0x00;
    };
}


void print_result ( char *s ) {
    cur.pos_x = 40;
    MZ800_INVPRNon ();
    prnc ( '[' );
    msg ( s );
    prnc ( ']' ); 
    MZ800_INVPRNoff ();
}


void System_serviceDI ( void ) __naked {
    __asm
    
    di
    ret
    
    __endasm;
}




void System_serviceEI ( void ) __naked {
    __asm
    
    ei
    ret
    
    __endasm;
}


void main ( void ) {

#ifdef USE_WRITE_TEST
    char tsttxt[] = "NAZDAR BAZAR!";
    char *c = tsttxt;
    unsigned int addr;
#endif

    unsigned char drive_id = FDC_DRIVE0;

    FDC_DRIVE *drive; 
    
    FDC_VERSION *version;
    
    FDC_TIMERS *fdct;

    unsigned char track = 1;
    unsigned char sector = 1;
    unsigned char drive_params = FDC_SECTOR_SIZE_256 | FDC_MEDIUM_SIDES2 | FDC_MEDIUM_NOT_CPL;
    
    void *dma = (void*) 0x6000;
    
    char err;
    
    xputc = prnc;
    xputs = msg;

    _FDC_serviceDI = System_serviceDI;
    _FDC_serviceEI = System_serviceEI;

    rBCOLOR = BLACK;
    rPAL = PAL0 | BLACK;
    rPAL = PAL1 | LWHITE;
    rPAL = PAL2 | WHITE;
    rPAL = PAL3 | BLACK;
    
    rWF = PLANE_I | PLANE_III;
    video_cls ();
    rWF = PLANE_I;
    MZ800_INVPRNoff ();
    cur.pos_x = 0;
    cur.pos_y = 0;



    /* mame pripojen radic? */
    xprintf ( "FDC test:" );

    if ( 0 == testFDC_isPresent () ) {
        print_result ( "OK" );
    } else {
        print_result ( "NOT FOUND" );
        // radic nenalezen, nema smysl dal pokracovat
        while ( 1 ) {};
    };




    /* info o driveru */
    cur.pos_x = 0;
    cur.pos_y += 2;
    
    version = FDC_serviceVersion ();
    
    xprintf ( "FDC driver:" );
    
    cur.pos_x = 40;
    cur.pos_y ++;
        
    xprintf ( "info  = %s", &version->description );

    cur.pos_x = 40;
    cur.pos_y ++;
    
    xprintf ( "major = 0x%04x", version->major );

    cur.pos_x = 40;
    cur.pos_y ++;

    xprintf ( "minor = 0x%04x", version->minor );


    /* info o timingu */
    cur.pos_x = 0;
    cur.pos_y += 2;
    
    fdct = FDC_serviceTimers ();
    
    xprintf ( "FDC timing:" );
    
    cur.pos_x = 40;
    cur.pos_y ++;
        
    xprintf ( "motor = %d ms", fdct->driveOnTime );

    cur.pos_x = 40;
    cur.pos_y ++;
    
    xprintf ( "track = %d ms", fdct->trackTime );

    cur.pos_x = 40;
    cur.pos_y ++;

    xprintf ( "step  = %d ms", fdct->stepTime );

    cur.pos_x = 40;
    cur.pos_y ++;

    xprintf ( "off   = %d calls", fdct->motorPredef );




    /* vybereme a nastavime disk */
    cur.pos_x = 0;
    cur.pos_y += 2; 
    xprintf ( "Select FD%x:", drive_id );


    drive = FDC_serviceSeldsk ( drive_id );
    *drive = drive_params;

    print_result ( "SELECTED" );



    /* nastavime parametry cteni */
    cur.pos_x = 0;
    cur.pos_y++; 
    xprintf ( "Set operating params:" );

    FDC_serviceSettrk ( track );
    FDC_serviceSetsec ( sector );
    FDC_serviceSetdma ( dma );

    print_result ( "OK" );



    /* cteni sektoru */
    cur.pos_x = 0;
    cur.pos_y += 2; 
    xprintf ( "Read sectors:" );
    
    cur.pos_x = 0;
    cur.pos_y++;
    xprintf ( "dma=0x%04x, trk=%d, sec=%d, drvprm=0x%02x:", (unsigned int) dma, track, sector, drive_params );

    err = FDC_serviceRead ();
    if ( err ) prg_error ( err );
    print_result ( "OK" );


    cur.pos_x = 0;
    cur.pos_y++;
    xprintf ( "dma=0x%04x, trk=%d, sec=%d, drvprm=0x%02x:", ((unsigned int) dma + 0x1000), track, (sector + 1), drive_params );

    FDC_serviceSetsec ( sector + 1 );
    FDC_serviceSetdma ( (void *) ( (unsigned int) dma + 0x1000 ) );

    err = FDC_serviceRead ();
    if ( err ) prg_error ( err );    
    print_result ( "OK" );


#ifdef USE_WRITE_TEST
    /* zapiseme do pameti vtipnou hlasku */
    addr = (unsigned int) dma;
    
    while ( c[0] != '\0' ) {
        * (char *) addr++ = c[0];
        c++;
    };



    /* zapis sektoru */
    cur.pos_x = 0;
    cur.pos_y += 2; 
    xprintf ( "Write sector:" );


    cur.pos_x = 0;
    cur.pos_y++;
    xprintf ( "dma=0x%04x, trk=%d, sec=%d, drvprm=0x%02x:", (unsigned int) dma, track, (sector + 1), drive_params );

    FDC_serviceSetsec ( sector + 1 );

    FDC_serviceSetdma ( dma );

    err = FDC_serviceWrite ();
    if ( err ) prg_error ( err );
    print_result ( "OK" );
#endif


    /* konec */
    FDC_serviceMotorOff ();
    

    cur.pos_x = 0;
    cur.pos_y += 2; 
    xprintf ( "Done." );
    
    
    while ( 1 ) {};
}
